package org.kisti.htc.myproxy;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.Scanner;

import javax.crypto.BadPaddingException;
import javax.swing.JOptionPane;

import org.globus.gsi.CertUtil;
import org.globus.gsi.GSIConstants;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.OpenSSLKey;
import org.globus.gsi.X509ExtensionSet;
import org.globus.gsi.bc.BouncyCastleCertProcessingFactory;
import org.globus.gsi.bc.BouncyCastleOpenSSLKey;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.myproxy.CredentialInfo;
import org.globus.myproxy.InfoParams;
import org.globus.myproxy.MyProxy;
import org.globus.tools.ui.proxy.GridProxyProperties;
import org.globus.util.Util;
import org.ietf.jgss.GSSCredential;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HTCaaS_ProxyInfo extends gridPropertiesUtil{

		public MyProxy myproxy;   
	    private GlobusCredential gridProxy = null;
	    private GSSCredential credential = null;
	    
	    private static String username;
	    private static String password;
	    static Scanner in = new Scanner(System.in);
	    final static Logger logger = LoggerFactory.getLogger(HTCaaS_ProxyInfo.class);	    	
	
	    public HTCaaS_ProxyInfo(GridProxyProperties gpp, String un, String pw){
	    	super();
	    	this.gridProps = gpp;
	    	this.username= un;	
	    	this.password = pw;
	    }
	
	    public void Info(){
	    	  	
	    	 if(createProxy(gridProps.getCredLifetime())){
			    	 InfoParams infoRequest = new InfoParams();
			         infoRequest.setUserName( username.trim());
			         infoRequest.setPassphrase("DUMMY-PASSPHRASE");
	    	 
	    	String tmp;
	    	
	    	try{
	    	   		myproxy = new MyProxy(gridProps.getMyproxyServer(),gridProps.getMyproxyPort());
	    	   		
	        	 if(gridProxy.getSubject()!= null){
	        	 //TODO   ServerDN Constant 생성 해줘야함 
	        	 myproxy.setAuthorization(new org.globus.gsi.gssapi.auth.IdentityAuthorization("/C=KR/O=KISTI/O=GRID/O=KISTI/CN=host/"+gridProps.getMyproxyServer()));
	        	 CredentialInfo[] info = myproxy.info(new GlobusGSSCredentialImpl(gridProxy,
	                     GSSCredential.INITIATE_AND_ACCEPT),infoRequest);
	        	 
	        	 System.out.println ("From MyProxy server: " + myproxy.getHost());
	             System.out.println ("Owner: " + info[0].getOwner());
	             
	             for (int i=0;i<info.length;i++) {
	            	 
	                 tmp = info[i].getName();
	                 System.out.println ((tmp == null) ? "Default:" : tmp +":");
	                 System.out.println ("\tStart Time  : " + 
	                		  info[i].getStartTimeAsDate());
	                 System.out.println ("\tEnd Time    : " + 
	                		 info[i].getEndTimeAsDate());

	                 long now = System.currentTimeMillis();
	                 
	                 if (info[i].getEndTime() > now) {
	                     System.out.println ("\tTime left   : " +
	                                         Util.formatTimeSec((info[i].getEndTime() - now)/1000));
	                 } else {
	                     System.out.println ("\tTime left   : expired");
	                 }

	                 tmp = info[i].getRetrievers();
	                 if (tmp != null) {
	                     System.out.println ("\tRetrievers  : "+tmp);
	                 }
	                 tmp = info[i].getRenewers();
	                 if (tmp != null) {
	                     System.out.println ("\tRenewers    : "+tmp);
	                 }
	                 tmp = info[i].getDescription();
	                 if (tmp != null) {
	                     System.out.println ("\tDescription : "+tmp);
	                 }
	             }
	        	 }        	 
	         }catch(Exception e){  logger.error( "Failed to get info from myproxy server!\n\"" + e.getMessage());	  }
	    	
	    	 }else if (gridProxy == null ){  
	    		logger.info("GridProxy not exists"); 
	    		return; 
	        }else {
	        	logger.error("Failed to get Proxy Info ");
	        }
	    	 
	    }
	
	    public boolean createProxy(int hours) {      
	        X509Certificate userCert = null;
	        PrivateKey userKey = null;
	        char[] pwd;
	        FileOutputStream out = null;
	        try {
	            userCert = CertUtil.loadCertificate(gridProps.getUserCertFile());

	            if (userCert == null) {
	               logger.error("Failed to load cert: {} ", gridProps.getUserCertFile());
	                return false;
	            }
	            //password = "DUMMY-PASSPHRASE";
	            pwd = password.toCharArray();

	            OpenSSLKey key = new BouncyCastleOpenSSLKey(gridProps
	                    .getUserKeyFile());

	            if (key.isEncrypted()) {
	                key.decrypt(new String(pwd));
	            }

	            userKey = key.getPrivateKey();

	            pwd = null;

	            if (userKey == null) {
//	                JOptionPane.showMessageDialog(this,
//	                        "Failed to load private key: "
//	                                + gridProps.getUserKeyFile(), "Error",
//	                        JOptionPane.ERROR_MESSAGE);
	            	
	            	
	                return false;
	            }

	            BouncyCastleCertProcessingFactory factory = 
	                BouncyCastleCertProcessingFactory
	                .getDefault();

	            int proxyType = (gridProps.getLimited()) ? 
	                GSIConstants.GSI_4_LIMITED_PROXY
	                : GSIConstants.GSI_4_IMPERSONATION_PROXY;

	            X509ExtensionSet extSet = null;

	            gridProxy = factory.createCredential(
	                    new X509Certificate[] { userCert }, userKey, gridProps
	                            .getBits(), hours*3600, proxyType, extSet);

	            if (gridProxy == null) {
	            	logger.error("Failed to create proxy! ");

	                return false;
	            }
	            
	            File file = Util.createFile(gridProps.getProxyFile());
	            // set read only permissions
	            if (!Util.setOwnerAccessOnly(gridProps.getProxyFile())) {

	            }
	            out = new FileOutputStream(file);
	            gridProxy.save(out);

	        } catch (BadPaddingException e){
	        	System.out.println("You have entered an incorrect password");
	        } catch (Exception e) {
	            e.printStackTrace();
	            logger.info( "Failed to create proxy!\n\" {}", e.getMessage() );
	           
	            return false;
	        } finally {
	            if (out != null) {
	                try {
	                    out.close();
	                } catch (IOException e) {
	                }
	            }
	        }
	        return true;
	    }
	
	
	public static void main(String[] args) {
		
		String un = "seoyoung";
		String pw = "sy1357";
		GridProxyProperties gp = new GridProxyProperties();
		gp.setMyproxyServer("amga.kisti.re.kr");
		HTCaaS_ProxyInfo pinfo = new  HTCaaS_ProxyInfo(gp,un, pw);
		pinfo.Info();

	}

}
