package org.kisti.htc.myproxy;

import java.io.ByteArrayInputStream;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.logging.Logger;

/**
 * <p>Created by Jeff Gaynor<br>
 * on Jun 15, 2010 at  2:45:47 PM
 */
public class MyProxyUtil {
    static Logger logger;
    /**
     * This required by the specification
     */
    public static final String BEGIN_CERTIFICATE = "-----BEGIN CERTIFICATE-----";
    public static final String END_CERTIFICATE = "-----END CERTIFICATE-----";
    public static final String BEGIN_RSA_PRIVATE_KEY = "-----BEGIN RSA PRIVATE KEY-----";
    public static final String END_RSA_PRIVATE_KEY = "-----END RSA PRIVATE KEY-----";
    

    public static Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger(MyProxyUtil.class.getName());
        }
        return logger;
    }

    /**
     * A utility that will take a list of Base64 encoded X509 certificates and convert them into
     * an array of java.security.cert.X509Certificate objects.
     * @param certList
     * @param nameList The array of file names corresponding to each cert. This is only used to generate error messages
     *             and if it is omitted will not otherwise effect function.
     * @return
     * @throws CertificateException
     */
    public static X509Certificate[] getX509CertsFromStringList(
            String[] certList, String[] nameList) throws CertificateException {
        CertificateFactory certFactory = CertificateFactory
                .getInstance("X.509");
        Collection<X509Certificate> c = new ArrayList<X509Certificate>(
                certList.length);
        for (int i = 0; i < certList.length; i++) {
            int index = -1;
            String certData = certList[i];
            if (certData != null) {
                index = certData.indexOf(BEGIN_CERTIFICATE);
            }
            if (index >= 0) {
                certData = certData.substring(index);
                ByteArrayInputStream inputStream = new ByteArrayInputStream(
                        certData.getBytes());
                try {
                    X509Certificate cert = (X509Certificate) certFactory
                            .generateCertificate(inputStream);
                    c.add(cert);
                } catch (Exception e) {
                    if (nameList != null) {
                        getLogger().warning(nameList[i]
                                + " can not be parsed as an X509Certificate.");
                    } else {
                        getLogger().warning("failed to parse an X509Certificate");
                    }
                }
            }
        }
        if (c.isEmpty())
            return null;
        return c.toArray(new X509Certificate[0]);
    }
}
